package juego;

import java.io.IOException;

import javax.swing.JOptionPane;

import stratego.Tablero;
import stratego.Motor;
import stratego.Movimiento;
import stratego.Pieza;
import stratego.Ajustes;
import stratego.Puntero;
import stratego.Estados;



public class IAMotor extends Motor implements CompControles, ControlesUsuario
{
	private Vista view = null;
	private IA ai = null;
	private int userColor = 0;
	
	public IAMotor(Vista v)
	{
		view = v;
		board = new Tablero();
		ai = new IA(board, this);
	}
	
	public void play()
	{
		if (status != Estados.SETUP)
			return;
		
		if (board.getTraySize() == 0)
		{
			status = Estados.PLAYING;
			if (turn!=userColor)
				requestCompMove();
		}
		else 
		{
			try {
				ai.getBoardSetup();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		if (Ajustes.bShowAll)
			board.showAll();
	}
	
	private void requestCompMove()
	{
		if (status == Estados.PLAYING)
		if (turn != userColor)
			ai.getMove();
	}	
	
	public void userRequestMove(Movimiento m)
	{
		if (m == null) return;
		
		if (status == Estados.SETUP)
		{
			if (setupRemovePiece(m.getFrom()))
					setupPlacePiece(m.getPiece(), m.getTo());
		}
		else
		{
			if (turn != userColor)
			{
				if (!IA.aiLock.isLocked())
					requestCompMove();
			}
			else
			{
				if (Ajustes.bShowAll)
					board.showAll();
				else if (!Ajustes.bNoHideAll)
					board.hideAll();
				
				if (requestMove(m))
					requestCompMove();
			}
		}
	}

	public void aiReturnMove(Movimiento m)
	{
		if (turn == userColor)
			return;
		
		if (m==null || m.getPiece()==null || m.getFrom()==null || m.getTo()==null)
		{
			//IA atrapado
			status = Estados.STOPPED;
			board.showAll();
			view.gameOver(userColor);
			return;
		}
		if (m.getPiece().getColor() == userColor)
			return; //Debera suceder de todos modos

		requestMove(m);
		if (turn != userColor)
			JOptionPane.showMessageDialog(null,
					"IA Error" , "Error Crtico", JOptionPane.ERROR_MESSAGE);
	}
	
	public void aiReturnPlace(Pieza p, Puntero s)
	{
		if (!setupPlacePiece(p, s))
			return;
		view.update();
	}
	 
	@Override
	protected void gameOver(int winner)
	{
		view.gameOver(winner);
	}

	@Override
	protected void update()
	{
		view.update();
	}
		
	public void newGame(int c)
	{
	    super.newGame();
	    this.userColor = c;
	    this.board.setTopColor((this.userColor + 1) % 2);
	    this.ai.setCompColor((this.userColor + 1) % 2);
	}
	
	/*public void demo()
	{
		if (status != Estados.SETUP)
			return;
		
		Ajustes.bShowAll = true;
		
		if (board.getTraySize() == 0)
		{
			status = Estados.PLAYING;
			if (turn!=userColor)
				requestCompMove();
		}
		else 
		{
			try {
				ai.getBoardSetup();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	public void demoGame(int c)
	{
	    super.demoGame();
	    this.userColor = c;
	    this.board.setTopColor((this.userColor + 1) % 2);
	    this.ai.setCompColor((this.userColor + 1) % 2);
	}*/
}
