package servidor;

import java.io.IOException;
import java.util.ArrayList;
import java.util.concurrent.Semaphore;
import java.util.concurrent.locks.ReentrantLock;

public class Servidor
{
	private ArrayList<Juego> games = new ArrayList<Juego>();
	private ReentrantLock gamesLock = new ReentrantLock();
	private Semaphore ignore = new Semaphore(1);
	private ServidorControlador controller = null;
	private Servidor me = null;
	private Juego active = null;
	private boolean running;
	private boolean closing;
	private int port;
	
	private Thread main = new Thread()
	{
		public void run()
		{
			while(running)
			{
				
				try
				{
					ignore.acquireUninterruptibly();
					ignore.release();
					
					active = new Juego(port, me);
					gamesLock.lock();
					{
						games.add(active);
					}
					gamesLock.unlock();
					active.start();
					active = null;
				}
				catch (IOException e)
				{
					controller.print("Puerto no disponible, por favor selecciona un puerto diferente.");
					controller.print("Ignorando nuevas conexiones.");
					ignore(true);
				}
			}
		}
	};
	
	public Servidor(int p, ServidorControlador sc)
	{
		port = p;
		me = this;
		controller = sc;
		running = true;
		closing = false;
		main.start();
	}
	
	public void gameOver(Juego s)
	{
		gamesLock.lock();
		{
			games.remove(s);
		}
		gamesLock.unlock();
		
		if (closing && games.size() <= 1)
		{
			close();
			System.exit(0);
		}
	}
	
	public void ignore(boolean b)
	{
		if (b == (0 == ignore.availablePermits()))
			return;
		
		if (0 == ignore.availablePermits())
		{
			ignore.release();
		}
		else
		{
			ignore.acquireUninterruptibly();
			
			if (active != null)
				active.disconnect();
		}
	}
	
	public boolean isIgnoring()
	{
		return 0 == ignore.availablePermits();
	}
	
	public void closeWhenEmpty(boolean b)
	{
		closing = b;

		if (closing && (games.size() <= 1))
		{
			close();
			System.exit(0);
		}
	}
	
	public boolean isClosingWhenEmpty()
	{
		return closing;
	}

	public int getPort()
	{
		return port;
	}
	
	public void setPort(int p)
	{
		port = p;			
		try
		{
			if (active != null)
				active.setPort(port);
		}
		catch (Exception e){}
	}
	
	@SuppressWarnings("deprecation")
	public void close()
	{
		running = false;
		if (isIgnoring())
			main.stop(); //No preocuparse, no tenemos en cuenta los recursos
			
		gamesLock.lock();
		{
			while (games.size() > 0)
				games.get(0).disconnect();
		}
		gamesLock.unlock();
		
		try
		{
			main.join();
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
	}
	
	public String games()
	{
		String ret = "";
		gamesLock.lock();
		{
			for (int i=0;i<games.size();i++)
			{
				ret += "Juego " + (i+1) + "\n";
				ret += games.get(i).status() + "\n\n";
			}
		}
		gamesLock.unlock();
		return ret;
	}
	
	public String status()
	{
		return games.size() + " Juegos activados";
	}
}
