package stratego;

import java.util.ArrayList;
import java.util.Collections;

public class Tablero
{
	protected static class UniqueID
	{
		private static int id = 0;
		
		static public int get()
		{
			id++;
			return id;
		}
	}
	
	public static final int RED  = 0;
	public static final int BLUE = 1;
	public static final Puntero IN_TRAY = new Puntero(-1, -1);
	public static final Pieza INVALID = new Pieza(-1, -1, Rango.WATER);
	
	protected Pieza[][] grid = new Pieza[10][10];
	protected ArrayList<Pieza> tray = new ArrayList<Pieza>();
	protected ArrayList<Movimiento> recentMoves = new ArrayList<Movimiento>();
	private int topColor;
	
	public Tablero()
	{
		//Crear lagos
		grid[2][4] = new Pieza(UniqueID.get(), -1, Rango.WATER);
		grid[3][4] = new Pieza(UniqueID.get(), -1, Rango.WATER);
		grid[2][5] = new Pieza(UniqueID.get(), -1, Rango.WATER);
		grid[3][5] = new Pieza(UniqueID.get(), -1, Rango.WATER);
		grid[6][4] = new Pieza(UniqueID.get(), -1, Rango.WATER);
		grid[7][4] = new Pieza(UniqueID.get(), -1, Rango.WATER);
		grid[6][5] = new Pieza(UniqueID.get(), -1, Rango.WATER);
		grid[7][5] = new Pieza(UniqueID.get(), -1, Rango.WATER);
		
		//Crear piezas
		for (int i=0;i<2;i++)
		{
			tray.add(new Pieza(UniqueID.get(), i, Rango.FLAG));
			tray.add(new Pieza(UniqueID.get(), i, Rango.SPY));
			tray.add(new Pieza(UniqueID.get(), i, Rango.ONE));
			tray.add(new Pieza(UniqueID.get(), i, Rango.TWO));
			for (int j=0;j<2;j++)
				tray.add(new Pieza(UniqueID.get(), i, Rango.THREE));
			for (int j=0;j<3;j++)
				tray.add(new Pieza(UniqueID.get(), i, Rango.FOUR));
			for (int j=0;j<4;j++)
				tray.add(new Pieza(UniqueID.get(), i, Rango.FIVE));
			for (int j=0;j<4;j++)
				tray.add(new Pieza(UniqueID.get(), i, Rango.SIX));
			for (int j=0;j<4;j++)
				tray.add(new Pieza(UniqueID.get(), i, Rango.SEVEN));
			for (int j=0;j<5;j++)
				tray.add(new Pieza(UniqueID.get(), i, Rango.EIGHT));
			for (int j=0;j<8;j++)
				tray.add(new Pieza(UniqueID.get(), i, Rango.NINE));
			for (int j=0;j<6;j++)
				tray.add(new Pieza(UniqueID.get(), i, Rango.BOMB));
		}
		Collections.sort(tray);
	}
	
	public Tablero(Tablero b)
	{
		for (int i=0;i<10;i++)
		for (int j=0;j<10;j++)
			grid[i][j] = b.getPiece(i, j);
			
		tray.addAll(b.tray);
		recentMoves.addAll(b.recentMoves);
	}

	public boolean add(Pieza p, Puntero s)
	{
		if (p.getColor() == topColor)
		{
			if(s.getY() > 3)
				return false;
		}
		else
		{
			if (s.getY() < 6)
				return false;
		}
			
		if (getPiece(s) == null)
		{
			setKnown(p, false);
			grid[s.getX()][s.getY()] = p;
			tray.remove(p);
			return true;
		}
		return false;
	}
	
	public boolean attack(Movimiento m)
	{
		if (validAttack(m))
		{
			setShown(m.getPiece(), true);
			if (!Ajustes.bNoShowDefender || m.getPiece().getRank().equals(Rango.NINE))
				setShown(getPiece(m.getTo()), true);
			
			int result = winFight(m.getPiece(), getPiece(m.getTo()));
			if (result == 1)
			{
				remove(m.getTo());
				setPiece(m.getPiece(), m.getTo());
				setPiece(null, m.getFrom());
			}
			else if (result == -1)
			{
				remove(m.getFrom());
				remove(m.getTo());
			}
			else 
			{
				clearRecentMoves(getPiece(m.getTo()));
				
				if (Ajustes.bNoMoveDefender ||
						getPiece(m.getTo()).getRank().equals(Rango.BOMB))
					remove(m.getFrom());
				else if (m.getPiece().getRank().equals(Rango.NINE))
					scoutLose(m);
				else
				{
					remove(m.getFrom());
					setPiece(getPiece(m.getTo()), m.getFrom());
					setPiece(null, m.getTo());
				}
			}
			return true;
		}
		return false;
	}

	public int checkWin()
	{
		int flags = 0;
		int flagColor = -1;
		for (int i=0;i<10;i++)
		for (int j=0;j<10;j++)
			if (getPiece(i, j) != null)
			if (getPiece(i, j).getRank().equals(Rango.FLAG))
			{
				flagColor = grid[i][j].getColor();
				flags++;
			}
		if (flags!=2)
			return flagColor;
		
		for (int k=0;k<2;k++)
		{
			int movable = 0;
			for (int i=0;i<10;i++)
			for (int j=0;j<10;j++)
			{
				if (getPiece(i, j) != null)
				if (getPiece(i, j).getColor() == k)
				if (!getPiece(i, j).getRank().equals(Rango.FLAG))
				if (!getPiece(i, j).getRank().equals(Rango.BOMB))
				if (getPiece(i+1, j) == null ||
					getPiece(i+1, j).getColor() == (k+1)%2||
					getPiece(i, j+1) == null ||
					getPiece(i, j+1).getColor() == (k+1)%2||
					getPiece(i-1, j) == null ||
					getPiece(i-1, j).getColor() == (k+1)%2||
					getPiece(i, j-1) == null ||
					getPiece(i, j-1).getColor() == (k+1)%2)
					
					movable++;
			}
			
			if (movable==0)
				return (k+1)%2;
		}
		
		return -1;
	}
	
	public void clear()
	{
		for (int i=0;i<10;i++)
		for (int j=0;j<10;j++)
			if (grid[i][j] != null)
			if (!grid[i][j].getRank().equals(Rango.WATER))
			{
				tray.add(grid[i][j]);
				grid[i][j] = null;
			}
		
		Collections.sort(tray);
		
		for (Pieza p: tray)
			setKnown(p, false);
	}
	
	private void clearRecentMoves(Pieza p)
	{
		for (int i=0;i<recentMoves.size();i++)
			if (recentMoves.get(i).getPiece() == p)
				recentMoves.remove(i);
				
	}

	public Pieza getPiece(int x, int y)
	{
		if (x<0||x>9||
			y<0||y>9)
			return INVALID;
		return grid[x][y];
	}
	
	public Pieza getPiece(Puntero s)
	{
		if (s.getX()<0||s.getX()>9||
			s.getY()<0||s.getY()>9)
			return INVALID;
		return grid[s.getX()][s.getY()];
	}

	
	public Pieza getTrayPiece(int i)
	{
		return tray.get(i);
	}
	
	public int getTraySize()
	{
		return tray.size();
	}
	
	public void showAll()
	{
		for (int i=0;i<10;i++)
		for (int j=0;j<10;j++)
			if (grid[i][j] != null)
				setShown(grid[i][j], true);
		for (Pieza p: tray)
			setShown(p, true);
	}
	
	public void hideAll()
	{
		for (int i=0;i<10;i++)
		for (int j=0;j<10;j++)
			if (grid[i][j] != null)
				setShown(grid[i][j], false);

		hideTray();
	}
	
	public void hideTray()
	{
		for (Pieza p: tray)
			setShown(p, false);
	}
	
	protected void setKnown(Pieza p, boolean b)
	{
		p.setKnown(b);
	}
	
	protected void setPiece(Pieza p, Puntero s)
	{
		grid[s.getX()][s.getY()] = p;
	}
	
	protected void setShown(Pieza p, boolean b)
	{
		p.setShown(b);
	}
	
	public void setTopColor(int c)
	{
		topColor = c;
	}
	
	public boolean move(Movimiento m)
	{
		if (validMove(m))
		{
			recentMoves.add(m);
			if (recentMoves.size() > 4)
				recentMoves.remove(0);
			
			setPiece(m.getPiece(), m.getTo());
			setPiece(null, m.getFrom());
			return true;
		}
		
		return false;
	}
	
	public boolean remove(Puntero s)
	{
		if (getPiece(s) == null)
			return false;
		if (getPiece(s) == INVALID)
			return false;
		
		setShown(getPiece(s), true);
		tray.add(getPiece(s));
		setPiece(null, s);
		Collections.sort(tray);
		return true;
	}
	
	private void scoutLose(Movimiento m)
	{
		Puntero tmp = getScoutLooseFrom(m);

		remove(m.getFrom());
		setPiece(getPiece(m.getTo()), tmp);
		setPiece(null, m.getTo());
	}
	
	private Puntero getScoutLooseFrom(Movimiento m)
	{
		if (m.getFrom().getX() == m.getTo().getX())
		{
			if (m.getFrom().getY() < m.getTo().getY())
				return new Puntero(m.getTo().getX(), m.getTo().getY() - 1);
			else
				return new Puntero(m.getTo().getX(), m.getTo().getY() + 1);
		}
		else //if (m.getFrom().getY() == m.getTo().getY())
		{
			if (m.getFrom().getX() < m.getTo().getX())
				return new Puntero(m.getTo().getX() - 1, m.getTo().getY());
			else
				return new Puntero(m.getTo().getX() + 1, m.getTo().getY());
		}
	}
	
	protected boolean validAttack(Movimiento m)
	{
		if (m.getTo() != IN_TRAY)
			if (m.getTo().getX()<0||m.getTo().getX()>9||
				m.getTo().getY()<0||m.getTo().getY()>9)
				return false;
		if (getPiece(m.getTo()) == null)
			return false;
		if (getPiece(m.getFrom()) == null)
			return false;
		if (getPiece(m.getTo()).getRank().equals(Rango.WATER))
			return false;
		if (getPiece(m.getFrom()) != m.getPiece())
			return false;
		if (m.getPiece().getColor() == getPiece(m.getTo()).getColor())
			return false;
		

		Pieza tmp = getPiece(m.getTo());
		setPiece(null, m.getTo());
		
		boolean valid = validMove(m);
		setPiece(tmp, m.getTo());
		
		return valid;

	}
	
	protected boolean validMove(Movimiento m)
	{
		if (m.getTo() != IN_TRAY)
			if (m.getTo().getX()<0||m.getTo().getX()>9||
				m.getTo().getY()<0||m.getTo().getY()>9)
				return false;
		if (getPiece(m.getTo()) != null)
			return false;
		if (getPiece(m.getFrom()) == null)
			return false;
		if (getPiece(m.getFrom()) != m.getPiece())
			return false;
		
		//Comprobar regla: "un jugador no puede mover su pieza en diagonal"
		if (recentMoves.contains(m))
			return false;

		switch (m.getPiece().getRank())
		{
		case FLAG:
		case BOMB:
			return false;
		default:
			break;
		}

		if (m.getFrom().getX() == m.getTo().getX())
			if (Math.abs(m.getFrom().getY() - m.getTo().getY()) == 1)
				return true;
		if (m.getFrom().getY() == m.getTo().getY())
			if (Math.abs(m.getFrom().getX() - m.getTo().getX()) == 1)
				return true;

		if (m.getPiece().getRank().equals(Rango.NINE))
			return validScoutMove(m.getFrom().getX(), m.getFrom().getY(), m.getTo(), m.getPiece());

		return false;
	}
	
	private boolean validScoutMove(int x, int y, Puntero to, Pieza p)
	{
		if ( !(grid[x][y] == null || p.equals(grid[x][y])) )
			return false;

		if (x == to.getX())
		{
			if (Math.abs(y - to.getY()) == 1)
			{
				p.setKnown(true); //Los exploradores se revelan moviendo mas de una posicion
				return true;
			}
			if (y - to.getY() > 0)
				return validScoutMove(x, y - 1, to, p);
			if (y - to.getY() < 0)
				return validScoutMove(x, y + 1, to, p);
		}
		else if (y == to.getY())
		{
			if (Math.abs(x - to.getX()) == 1)
			{
				p.setKnown(true);
				return true;
			}
			if (x - to.getX() > 0)
				return validScoutMove(x - 1, y, to, p);
			if (x - to.getX() < 0)
				return validScoutMove(x + 1, y, to, p);
		}

		return false;
	}

	private int winFight(Pieza attack, Pieza defend)
	{
		if (attack.getRank().equals(defend.getRank()))
		{
			if (Ajustes.bDefendAdvantage)
				return 0;
			else
				return -1;
		}
		if (defend.getRank().equals(Rango.FLAG))
			return 1;
		if (defend.getRank().equals(Rango.SPY))
			return 1;
		if (defend.getRank().equals(Rango.BOMB))
		{
			if (attack.getRank().equals(Rango.EIGHT))
				return 1;
			if (Ajustes.bOneTimeBombs)
				return -1;
			else
				return 0;
		}
		if (attack.getRank().equals(Rango.SPY))
		{
			if (defend.getRank().equals(Rango.ONE))
				return 1;
			return 0;
		}
		
		if (attack.getRank().toInt() < defend.getRank().toInt())
			return 1;
		return 0;
	}
}